@@ -17,11 +17,12 @@ from aggify.exceptions import (
 from aggify.types import QueryParams, CollectionType
 from aggify.utilty import (
     to_mongo_positive_index,
-    check_fields_exist,
+    validate_field_existence,
     replace_values_recursive,
     convert_match_query,
-    check_field_exists,
+    check_field_already_exists,
     get_db_field,
+    copy_class,
 )
 
 AggifyType = TypeVar("AggifyType", bound=Callable[..., "Aggify"])
@@ -56,9 +57,7 @@ class Aggify:
             base_model: The base model class.
         """
         # Create a separate copy of the main class for safety and flexibility
-        self.base_model = type(
-            "Aggify_base_model", base_model.__bases__, dict(base_model.__dict__)
-        )
+        self.base_model = copy_class(base_model)
         self.pipelines: List[Dict[str, Union[dict, Any]]] = []
         self.start = None
         self.stop = None
@@ -84,13 +83,13 @@ class Aggify:
         Returns:
             Aggify: Returns an instance of the Aggify class for potential method chaining.
         """
-
-        if all([i in kwargs.values() for i in [0, 1]]):
+        filtered_kwargs = dict(kwargs)
+        filtered_kwargs.pop("id", None)
+        if all([i in filtered_kwargs.values() for i in [0, 1]]):
             raise InvalidProjection()
 
         # Extract fields to keep and check if _id should be deleted
         to_keep_values = {"id"}
-        delete_id = kwargs.get("id") is not None
         projection = {}
 
         # Add missing fields to the base model
@@ -109,19 +108,21 @@ class Aggify:
         # Remove fields from the base model, except the ones in to_keep_values and possibly _id
         if to_keep_values != {"id"}:
             keys_for_deletion = self.base_model._fields.keys() - to_keep_values  # noqa
-            if delete_id:
-                keys_for_deletion.add("id")
             for key in keys_for_deletion:
                 del self.base_model._fields[key]  # noqa
         # Append the projection stage to the pipelines
         self.pipelines.append({"$project": projection})
-
         # Return the instance for method chaining
         return self
 
     @last_out_stage_check
-    def group(self, expression: Union[str, None] = "id") -> "Aggify":
-        if expression:
+    def group(self, expression: Union[str, Dict, List, None] = "id") -> "Aggify":
+        if isinstance(expression, list):
+            expression = {
+                field: f"${self.get_field_name_recursively(field)}"
+                for field in expression
+            }
+        if expression and not isinstance(expression, dict):
             try:
                 expression = "$" + self.get_field_name_recursively(expression)
             except InvalidField:
@@ -143,6 +144,7 @@ class Aggify:
     @last_out_stage_check
     def raw(self, raw_query: dict) -> "Aggify":
         self.pipelines.append(raw_query)
+        self.pipelines = self.__combine_sequential_matches()
         return self
 
     @last_out_stage_check
@@ -456,7 +458,9 @@ class Aggify:
 
         # Determine the data type based on the aggregation operator
         self.pipelines[-1]["$group"].update({annotate_name: {acc: value}})
-        self.base_model._fields[annotate_name] = field_type  # noqa
+        base_model_fields = self.base_model._fields  # noqa
+        if not base_model_fields.get(annotate_name, None):
+            base_model_fields[annotate_name] = field_type
         return self
 
     def __match(self, matches: Dict[str, Any]):
@@ -519,7 +523,9 @@ class Aggify:
         return merged_pipeline
 
     # check_fields_exist(self.base_model, let)  # noqa
-    def get_field_name_recursively(self, field: str) -> str:
+    def get_field_name_recursively(
+        self, field: str, base: Union[CollectionType, None] = None
+    ) -> str:
         """
         Recursively fetch the field name by following the hierarchy indicated by the field parameter.
 
@@ -535,19 +541,19 @@ class Aggify:
         """
 
         field_name = []
-        prev_base = self.base_model
+        base = self.base_model if not base else base
 
         # Split the field based on double underscores and process each item
         for index, item in enumerate(field.split("__")):
             # Ensure the field exists at the current level of hierarchy
-            check_fields_exist(prev_base, [item])  # noqa
+            validate_field_existence(base, [item])  # noqa
 
             # Append the database field name to the field_name list
-            field_name.append(get_db_field(prev_base, item))
+            field_name.append(get_db_field(base, item))
 
             # Move to the next level in the model hierarchy
-            prev_base = self.get_model_field(prev_base, item)
-            prev_base = prev_base.__dict__.get("document_type_obj", prev_base)
+            base = self.get_model_field(base, item)
+            base = base.__dict__.get("document_type_obj", base)
 
         # Join the entire hierarchy using dots and return
         return ".".join(field_name)
@@ -561,6 +567,7 @@ class Aggify:
         let: Union[List[str], None] = None,
         local_field: Union[str, None] = None,
         foreign_field: Union[str, None] = None,
+        raw_let: Union[Dict, None] = None,
     ) -> "Aggify":
         """
         Generates a MongoDB lookup pipeline stage.
@@ -577,26 +584,27 @@ class Aggify:
             localField and foreignField are not used.
             local_field (Union[str, None], optional): The local field to join on when let not provided.
             foreign_field (Union[str, None], optional): The foreign field to join on when let not provided.
+            raw_let (Union[Dict, None]): raw let
 
         Returns:
             Aggify: An instance of the Aggify class representing a MongoDB lookup pipeline stage.
         """
 
         lookup_stages = []
-        check_field_exists(self.base_model, as_name)  # noqa
+        check_field_already_exists(self.base_model, as_name)  # noqa
         from_collection_name = from_collection._meta.get("collection")  # noqa
 
-        if not let and not (local_field and foreign_field):
+        if not (let or raw_let) and not (local_field and foreign_field):
             raise InvalidArgument(
-                expected_list=[["local_field", "foreign_field"], "let"]
+                expected_list=[["local_field", "foreign_field"], ["let", "raw_let"]]
             )
-        elif not let:
+        elif not (let or raw_let):
             lookup_stage = {
                 "$lookup": {
                     "from": from_collection_name,
-                    "localField": get_db_field(self.base_model, local_field),  # noqa
-                    "foreignField": get_db_field(
-                        from_collection, foreign_field  # noqa
+                    "localField": self.get_field_name_recursively(local_field),  # noqa
+                    "foreignField": self.get_field_name_recursively(
+                        base=from_collection, field=foreign_field  # noqa
                     ),
                     "as": as_name,
                 }
@@ -605,11 +613,16 @@ class Aggify:
             if not query:
                 raise InvalidArgument(expected_list=["query"])
 
+            if let is None:
+                let = []
+
             let_dict = {
                 field: f"${get_db_field(self.base_model, self.get_field_name_recursively(field))}"  # noqa
                 for field in let
             }
 
+            let = list(raw_let.keys()) if let is [] else let
+
             for q in query:
                 # Construct the match stage for each query
                 if isinstance(q, Q):
@@ -628,6 +641,8 @@ class Aggify:
                     )
 
             # Append the lookup stage with multiple match stages to the pipeline
+            if raw_let:
+                let_dict.update(raw_let)
             lookup_stage = {
                 "$lookup": {
                     "from": from_collection_name,
@@ -640,7 +655,7 @@ class Aggify:
         self.pipelines.append(lookup_stage)
 
         # Add this new field to base model fields, which we can use it in the next stages.
-        self.base_model._fields[as_name] = from_collection  # noqa
+        self.base_model._fields[as_name] = copy_class(from_collection)  # noqa
 
         return self
 
@@ -678,13 +693,17 @@ class Aggify:
             InvalidEmbeddedField: If the specified embedded field is not found or is not of the correct type.
         """
         model_field = self.get_model_field(self.base_model, embedded_field)  # noqa
-
+        field_name = get_db_field(self.base_model, embedded_field)
+        if "__module__" in model_field.__dict__:
+            self.base_model._fields = (
+                model_field._fields
+            )  # load new fields into old model
+            return f"${field_name}"
         if not hasattr(model_field, "document_type") or not issubclass(
             model_field.document_type, EmbeddedDocument
         ):
             raise InvalidEmbeddedField(field=embedded_field)
-
-        return f"${model_field.db_field}"
+        return f"${field_name}"
 
     @last_out_stage_check
     def replace_root(
@@ -706,10 +725,13 @@ class Aggify:
         """
         name = self._replace_base(embedded_field)
 
-        if not merge:
-            new_root = {"$replaceRoot": {"$newRoot": name}}
-        else:
+        if merge:
             new_root = {"$replaceRoot": {"newRoot": {"$mergeObjects": [merge, name]}}}
+            self.base_model._fields.update(  # noqa
+                {key: mongoengine_fields.IntField() for key, value in merge.items()}
+            )
+        else:
+            new_root = {"$replaceRoot": {"newRoot": name}}
         self.pipelines.append(new_root)
 
         return self
@@ -738,6 +760,9 @@ class Aggify:
             new_root = {"$replaceWith": name}
         else:
             new_root = {"$replaceWith": {"$mergeObjects": [merge, name]}}
+            self.base_model._fields.update(  # noqa
+                {key: mongoengine_fields.IntField() for key, value in merge.items()}
+            )
         self.pipelines.append(new_root)
 
         return self
