@@ -1,4 +1,4 @@
-from typing import Type
+from typing import Type, List
 
 
 class AggifyBaseException(Exception):
@@ -28,12 +28,14 @@ class AnnotationError(InvalidPipelineStageError):
 
 class OutStageError(InvalidPipelineStageError):
     def __init__(self, stage):
-        self.message = f"You cannot add a {self!r} pipeline after $out stage!"
+        self.message = (
+            f"You cannot add a {self!r} pipeline after $out stage! stage : {stage}"
+        )
         super().__init__(self.message)
 
 
 class AggifyValueError(AggifyBaseException):
-    def __init__(self, expected_list: list[Type], result: Type):
+    def __init__(self, expected_list: List[Type], result: Type):
         self.message = (
             f"Input is not correctly passed, expected either of {[expected for expected in expected_list]}"
             f"but got {result}"
@@ -72,3 +74,9 @@ class InvalidArgument(AggifyBaseException):
         self.message = f"Input is not correctly passed, expected {[expected for expected in expected_list]}"
         self.expecteds = expected_list
         super().__init__(self.message)
+
+
+class InvalidProjection(AggifyBaseException):
+    def __init__(self):
+        self.message = "You can't use inclusion and exclusion together."
+        super().__init__(self.message)
